@@ -9,18 +9,21 @@ dataDM <- dataADaMCDISCP01$ADSL
 ## example of basic barplot:
 
 # treemap takes as input table with counts
-library(inTextSummaryTable)
+
+if (requireNamespace("inTextSummaryTable", quietly = TRUE)) {
 
 # total counts: Safety Analysis Set (patients with start date for the first treatment)
 dataTotal <- subset(dataDM, RFSTDTC != "")
 
 # compute adverse event table
-tableAE <- computeSummaryStatisticsTable(
+
+tableAE <- inTextSummaryTable::computeSummaryStatisticsTable(
 	data = dataAE,
 	rowVar = c("AEBODSYS", "AEDECOD"),
+	rowOrder = "total",
 	dataTotal = dataTotal,
 	labelVars = labelVars,
-	stats = getStats("count")
+	stats = inTextSummaryTable::getStats("count")
 )
 
 dataPlot <- subset(tableAE, AEDECOD != "Total")
@@ -44,16 +47,27 @@ barplotClinData(
 	labelVars = labelVars
 )
 
+# add a selection box
+if(interactive()){
+  barplotClinData(
+    data = dataPlot,
+    xVar = "AEDECOD", 
+    yVar = "n", yLab = "Number of patients with adverse events",
+    labelVars = labelVars,
+    selectVars = "AEBODSYS"
+  )
+}
+
 \dontrun{
 
 # display percentage of events per severity
-tableAEBySeverity <- computeSummaryStatisticsTable(
+tableAEBySeverity <- inTextSummaryTable::computeSummaryStatisticsTable(
 	data = dataAE,
 	rowVar = c("AEDECOD", "AESEV"),
 	dataTotal = dataTotal,
 	labelVars = labelVars,
 	statsPerc = "statm",
-	stats = getStats("%m"),
+	stats = inTextSummaryTable::getStats("%m"),
 	dataTotalPerc = dataAE,
 	rowVarTotalPerc = "AEDECOD"
 )
@@ -76,4 +90,6 @@ barplotClinData(
 	subtitle = "Group: severity"
 )
 
+}
+
 }
\ No newline at end of file
