@@ -1,6 +1,8 @@
 context("Render a clinical data review report")
 
 library(yaml)
+library(parallel)
+library(xml2)
 
 test_that("Report titles are extracted for each config file", {
       
@@ -9,7 +11,7 @@ test_that("Report titles are extracted for each config file", {
 	configFiles <- list.files(testPathConfig)
 	otherConfigs <- configFiles[!grepl("config.yml", configFiles)]
 			
-	reportTitles <- checkReportTitles(configFiles, configDir = testPathConfig)
+	reportTitles <- clinDataReview:::checkReportTitles(configFiles, configDir = testPathConfig)
 	expect_type(reportTitles, "character")
 	expect_length(reportTitles, length(otherConfigs))
 	expect_named(reportTitles, otherConfigs)
@@ -24,7 +26,7 @@ test_that("An error is generated if the multiple reports have the same titles",
 			
 	configFilesError <- c(configFiles[1], configFiles[1])
 	expect_error(
-		checkReportTitles(configFilesError, configDir = testPathConfig),
+	  clinDataReview:::checkReportTitles(configFilesError, configDir = testPathConfig),
 		"The title .+ is duplicated."
 	)
 	
@@ -39,7 +41,7 @@ test_that("A warning is generated if a config is generated if a config file does
 			
 	configFilesWarning <- c(configFiles, "config-ciao")
 	expect_warning(
-		checkReportTitles(configFilesWarning, configDir = testPathConfig),
+	  clinDataReview:::checkReportTitles(configFilesWarning, configDir = testPathConfig),
 		"Please check the spelling is correct"
 	)
       
@@ -198,44 +200,12 @@ test_that("A warning is generated if the general config file is not available",
       
 })
 
-test_that("The creation of html file from markdown files for specified config files is successful", {
-     
-	skip_if_not(
-		condition = rmarkdown::pandoc_available(), 
-		message = "pandoc is not available"
-	)
-	
-	# get example config files
-	testPathBase <- normalizePath(path = "../files")
-	testPathConfig <- file.path(testPathBase, "config")
-	testPathInterim <- file.path(testPathBase, "interim")
-	
-	# copy md file to temp directory
-	testDir <- tempfile("mdConversion")
-	dir.create(testDir)
-	file.copy(from = testPathInterim, to = testDir, recursive = TRUE)
-	interimDir <- file.path(testDir, basename(testPathInterim))
-	
-	htmlOutput <- convertMdToHtml(
-		inputDir = testDir,
-		outputDir = testDir,
-		intermediateDir = interimDir,
-		configDir = testPathConfig, 
-		mdFiles = NULL,
-		indexPath = "index.Rmd",
-		quiet = TRUE # suppress printing of pandoc cmd line
-	)
-	expect_type(htmlOutput, "character")
-	expect_true(file.exists(htmlOutput))
-	expect_equal(normalizePath(dirname(htmlOutput)), normalizePath(testDir))
-	
-})
-
 test_that("The creation of html file from the specified markdown files is successful", {
 			
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
 	skip_if_not(
-		condition = rmarkdown::pandoc_available(), 
-		message = "pandoc is not available"
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
 	)
 			
 	# get example config files
@@ -250,19 +220,21 @@ test_that("The creation of html file from the specified markdown files is succes
 	interimDir <- file.path(testDir, basename(testPathInterim))
 
 	# Md files
-	filePathMd <- list.files(pattern = "md", interimDir, full.names = TRUE)
+	filePathMd <- file.path(interimDir, "adverseEvents-division.md")
       
-	htmlOutput <- convertMdToHtml(
-		outputDir = testDir,
-		intermediateDir = interimDir,
-		configDir = testPathConfig, 
-		mdFiles = filePathMd,
-		indexPath = "index.Rmd",
-		quiet = TRUE # suppress printing of pandoc cmd line
+	expect_message(
+		htmlOutput <- clinDataReview:::convertMdToHtml(
+			mdFile = filePathMd, 
+			indexPath = file.path(testPathBase, "index.Rmd"), 
+			intermediateDir = interimDir,
+			outputDir = interimDir,
+			quiet = TRUE, # suppress printing of pandoc cmd line
+		),
+		"Convert the Markdown file.+to html.+"
 	)
 	expect_type(htmlOutput, "character")
 	expect_true(file.exists(htmlOutput))
-	expect_equal(normalizePath(dirname(htmlOutput)), normalizePath(testDir))
+	expect_equal(normalizePath(dirname(htmlOutput)), normalizePath(interimDir))
       
 })
 
@@ -361,6 +333,17 @@ test_that("Session informations are merged successfully", {
       
 })
 
+test_that("Session informations with different elements are merged successfully", {
+			
+	sessionInfos <- list(sessionInfo(), sessionInfo())
+	sessionInfos[[1]]$loadedOnly <- NULL
+	expect_silent(
+		sessionInfo <- do.call(clinDataReview:::merge.sessionInfo, sessionInfos)
+	)
+	expect_s3_class(sessionInfo, "sessionInfo")
+			
+})
+
 test_that("The export of empty session infos to Markdown returns an empty output", {
       
 	expect_null(clinDataReview:::exportSessionInfoToMd(sessionInfos = NULL))
@@ -386,6 +369,12 @@ test_that("Session infos are successfully exported to Markdown", {
 test_that("A clinical data report is created successfully via the specification of config files", {
 	
 	skip_on_cran()
+	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
 			
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
@@ -402,16 +391,14 @@ test_that("A clinical data report is created successfully via the specification
 	# copy configuration files
 	file.copy(from = testPathConfig, to = testDir, recursive = TRUE)
       
-	# progress messages during execution
-	expect_message(
-		output <- render_clinDataReviewReport(
-			configFiles = configFiles,
-			configDir = testPathConfig,
-			inputDir = testDir,
-			intermediateDir = file.path(testDir, "interim"),
-			outputDir = outputDir,
-			quiet = TRUE # suppress printing of pandoc cmd line
-		)
+	output <- render_clinDataReviewReport(
+		configFiles = configFiles,
+		configDir = testPathConfig,
+		inputDir = testDir,
+		intermediateDir = file.path(testDir, "interim"),
+		outputDir = outputDir,
+		quiet = TRUE, # suppress printing of pandoc cmd line
+		verbose = FALSE
 	)
 	expect_type(output, "character")
 	expect_match(output, "introduction")
@@ -426,6 +413,12 @@ test_that("A clinical data report is created successfully with a log file", {
 			
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	testPathConfig <- file.path(testPathBase, "config")
@@ -441,16 +434,12 @@ test_that("A clinical data report is created successfully with a log file", {
       
 	logPath <- file.path(testDir, "log.txt")
       
-	# progress messages during execution
-	expect_message(
-		output <- render_clinDataReviewReport(
-			configDir = testPathConfig,
-			inputDir = testDir,
-			intermediateDir = file.path(testDir, "interim"),
-			outputDir = file.path(testDir, "report"),
-			logFile = logPath,
-			quiet = TRUE # suppress printing of pandoc cmd line
-		)
+	output <- render_clinDataReviewReport(
+		configDir = testPathConfig,
+		inputDir = testDir,
+		intermediateDir = file.path(testDir, "interim"),
+		outputDir = file.path(testDir, "report"),
+		logFile = logPath
 	)
 	expect_type(output, "character")
 	expect_match(output, "introduction")
@@ -462,6 +451,12 @@ test_that("A clinical data report is created successfully via the specification
 
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	testPathConfig <- file.path(testPathBase, "config")
@@ -477,21 +472,19 @@ test_that("A clinical data report is created successfully via the specification
 	# copy configuration files
 	file.copy(from = testPathConfig, to = testDir, recursive = TRUE)
 	
-	# progress messages during execution
-	expect_message(
-		output <- render_clinDataReviewReport(
-			configFiles = NULL,
-			configDir = file.path(testDir, basename(testPathConfig)),
-			inputDir = testDir,
-			intermediateDir = file.path(testDir, "interim"),
-			outputDir = outputDir,
-			quiet = TRUE # suppress printing of pandoc cmd line
-		)
+	output <- render_clinDataReviewReport(
+		configFiles = NULL,
+		configDir = file.path(testDir, basename(testPathConfig)),
+		inputDir = testDir,
+		intermediateDir = file.path(testDir, "interim"),
+		outputDir = outputDir,
+		quiet = TRUE, # suppress printing of pandoc cmd line
+		verbose = FALSE
 	)
 	expect_type(output, "character")
 	expect_match(output, "introduction")
 	htmlFiles <- list.files(pattern = "html", path = outputDir)
-	sectionName <- checkReportTitles(configFiles, configDir = testPathConfig)
+	sectionName <- clinDataReview:::checkReportTitles(configFiles, configDir = testPathConfig)
 	sectionName <- gsub(" ", "-", sectionName)
 	expect_true(
 		any(grepl(sectionName, htmlFiles, ignore.case = TRUE))
@@ -503,6 +496,12 @@ test_that("A warning is generated if a specified config file doesn't exist", {
       
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -542,15 +541,13 @@ test_that("A warning is generated if a specified config file doesn't exist", {
 	configFiles <- basename(configFiles)
 	
 	expect_warning(
-		# progress messages during execution
-		expect_message(
-			output <- render_clinDataReviewReport(
-				configDir = testDir,
-				outputDir = testDir,
-				inputDir = testDir,
-				intermediateDir = testDir,
-				quiet = TRUE # suppress printing of pandoc cmd line
-			)
+		output <- render_clinDataReviewReport(
+			configDir = testDir,
+			outputDir = testDir,
+			inputDir = testDir,
+			intermediateDir = testDir,
+			quiet = TRUE, # suppress printing of pandoc cmd line
+			verbose = FALSE
 		),
 		"configFile2.* cannot be found"
 	)
@@ -563,6 +560,12 @@ test_that("A warning is generated if a template report is already available", {
 			
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -600,26 +603,23 @@ test_that("A warning is generated if a template report is already available", {
 	configFiles <- basename(configFiles)
 	
 	# progress messages during execution
-	expect_message(
-		firstRun <- render_clinDataReviewReport(
+	firstRun <- render_clinDataReviewReport(
+		configDir = testDir,
+		outputDir = testDir,
+		inputDir = testDir,
+		intermediateDir = testDir,
+		quiet = TRUE, # suppress printing of pandoc cmd line
+		verbose = FALSE
+	)
+	
+	expect_warning(
+		secondRun <- render_clinDataReviewReport(
 			configDir = testDir,
 			outputDir = testDir,
 			inputDir = testDir,
 			intermediateDir = testDir,
-			quiet = TRUE # suppress printing of pandoc cmd line
-		)
-	)
-	
-	expect_warning(
-		# progress messages during execution
-		expect_message(
-			secondRun <- render_clinDataReviewReport(
-				configDir = testDir,
-				outputDir = testDir,
-				inputDir = testDir,
-				intermediateDir = testDir,
-				quiet = TRUE # suppress printing of pandoc cmd line   
-			)
+			quiet = TRUE, # suppress printing of pandoc cmd line 
+			verbose = FALSE
 		),
 		"Document with similar name than specified template from"
 	)
@@ -632,6 +632,12 @@ test_that("A warning is generated if the creation of a chapter fails'", {
      
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -643,7 +649,13 @@ test_that("A warning is generated if the creation of a chapter fails'", {
       
 	# file 1
 	configFile1 <- file.path(testDir, "configFile1.yml")
-	write_yaml(x = list(template = "templateWithError.Rmd"), file = configFile1)
+	write_yaml(
+	  x = list(
+	    template = "templateWithError.Rmd", 
+	    sectionTitle = "Chapter with error"
+	 ), 
+    file = configFile1
+  )
 	
 	# create a Rmd file returning an error
 	cat(
@@ -668,20 +680,18 @@ test_that("A warning is generated if the creation of a chapter fails'", {
 	configFiles <- basename(configFiles)
 	
 	expect_warning(
-		expect_error(
-			# progress messages during execution
-			expect_message(
-				render_clinDataReviewReport(
-					configDir = testDir,
-					outputDir = testDir,
-					intermediateDir = testDir,
-					inputDir = testDir,
-					quiet = TRUE # suppress printing of pandoc cmd line   
-				)
-			)
+	    render_clinDataReviewReport(
+	      configDir = testDir,
+	      outputDir = testDir,
+	      intermediateDir = testDir,
+	      inputDir = testDir,
+	      quiet = TRUE, # suppress printing of pandoc cmd line 
+		  verbose = FALSE
 		),
-		"Rendering of the .+ report for config file: .+ failed, a report with only the section title is created."
-	)
+    	paste(
+		"Rendering of the .+ report for config file:.+configFile1.yml.+failed,",
+      	"a report with only the section title is created.+")
+  )
       
 })
 
@@ -689,6 +699,12 @@ test_that("A warning is generated if the template name is missing in a config fi
 			
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -722,15 +738,13 @@ test_that("A warning is generated if the template name is missing in a config fi
 	configFiles <- basename(configFiles)
 	
 	expect_warning(
-		# progress messages during execution
-		expect_message(	
-			res <- render_clinDataReviewReport(
-				configDir = testDir,
-				outputDir = testDir,
-				intermediateDir = testDir,
-				inputDir = testDir,
-				quiet = TRUE # suppress printing of pandoc cmd line  
-			)
+		res <- render_clinDataReviewReport(
+			configDir = testDir,
+			outputDir = testDir,
+			intermediateDir = testDir,
+			inputDir = testDir,
+			quiet = TRUE, # suppress printing of pandoc cmd line 
+			verbose = FALSE
 		),
 		"Template missing for config file: .+."
 	)
@@ -743,6 +757,12 @@ test_that("A warning is generated if the config parameters don't comply to the t
     
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -776,15 +796,13 @@ test_that("A warning is generated if the config parameters don't comply to the t
 	configFiles <- basename(configFiles)
 	
 	expect_warning(
-		# progress messages during execution
-		expect_message(
-			res <- render_clinDataReviewReport(
-				configDir = testDir,
-				outputDir = testDir,
-				intermediateDir = testDir,
-				inputDir = testDir,
-				quiet = TRUE # suppress printing of pandoc cmd line     
-			)
+		res <- render_clinDataReviewReport(
+			configDir = testDir,
+			outputDir = testDir,
+			intermediateDir = testDir,
+			inputDir = testDir,
+			quiet = TRUE, # suppress printing of pandoc cmd line     
+			verbose = FALSE
 		),
 		"The report for the config file: .+ is not created because the check of the parameters failed"
 	)
@@ -797,6 +815,12 @@ test_that("A warning is generated if a template is not available in the specifie
       
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -830,15 +854,13 @@ test_that("A warning is generated if a template is not available in the specifie
 	configFiles <- basename(configFiles)
 	
 	expect_warning(
-		# progress messages during execution
-		expect_message(	
-			res <- render_clinDataReviewReport(
-				configDir = testDir,
-				outputDir = testDir,
-				intermediateDir = testDir,
-				inputDir = testDir ,
-				quiet = TRUE # suppress printing of pandoc cmd line     
-			)
+		res <- render_clinDataReviewReport(
+			configDir = testDir,
+			outputDir = testDir,
+			intermediateDir = testDir,
+			inputDir = testDir ,
+			quiet = TRUE, # suppress printing of pandoc cmd line 
+			verbose = FALSE
 		),
 		"Template file: .+ not available in.*myPackage.+ package."
 	)
@@ -851,6 +873,12 @@ test_that("A warning is generated if no config parameters are available", {
       
 	skip_on_cran()
 	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
 	# get example config files
 	testPathBase <- normalizePath(path = "../files")
 	indexPath <- file.path(testPathBase, "index.Rmd")
@@ -882,21 +910,17 @@ test_that("A warning is generated if no config parameters are available", {
 	)
 	configFiles <- c(configFileGeneral, configFile1)
 	configFiles <- basename(configFiles)
-	
-	expect_error(          
-		expect_warning(
-			# progress messages during execution
-			expect_message(	
-				res <- render_clinDataReviewReport(
-					configDir = testDir,
-					outputDir = testDir,
-					intermediateDir = testDir,
-					inputDir = testDir,
-					quiet = TRUE # suppress printing of pandoc cmd line     
-				)
-			),
-			"No config parameter available, input parameters for the report are not checked."
-		)
+	     
+	expect_warning(
+		res <- render_clinDataReviewReport(
+			configDir = testDir,
+			outputDir = testDir,
+			intermediateDir = testDir,
+			inputDir = testDir,
+			quiet = TRUE, # suppress printing of pandoc cmd line     
+			verbose = FALSE
+		),
+		"No config parameter available, input parameters for the report are not checked."
 	)
       
 })
@@ -937,16 +961,14 @@ test_that("A warning is generated if some Markdown files are missing for convers
 	
 	expect_error(
 		expect_warning(
-			# progress messages during execution
-			expect_message(	
-				convertMdToHtml(
-					configDir = testDir,
-					outputDir = testDir,
-					intermediateDir = testDir,
-					inputDir = testDir,
-					mdFiles = NULL,
-					quiet = TRUE # suppress printing of pandoc cmd line  
-				)
+			postProcessReport(
+				configDir = testDir,
+				outputDir = testDir,
+				intermediateDir = testDir,
+				inputDir = testDir,
+				mdFiles = NULL,
+				quiet = TRUE, # suppress printing of pandoc cmd line  
+				verbose = FALSE
 			),
 			"Markdown file(s): .+ are missing, these files are ignored."
 		)
@@ -1020,3 +1042,585 @@ test_that("Config parameters with R code and lazy-evaluation are correctly evalu
 	expect_equal(paramEvaluated$nrowData, nrow(customData))
 			
 })
+
+test_that("A clinical data report is created successfully in parallel", {
+			
+	skip_on_cran()
+	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+	
+	testDir <- tempfile("report")
+	dir.create(testDir)
+	
+	# create multiple config files
+	configDir <- file.path(testDir, "config")
+	dir.create(configDir)
+	nChapters <- 4
+	configFiles <- file.path(configDir, 
+		paste0("config-division-", seq_len(nChapters), ".yml")
+	)
+	for(i in seq_len(nChapters)){
+		yaml::write_yaml(
+			list(
+				template = "divisionTemplate.Rmd",
+				templatePackage = "clinDataReview",
+				reportTitle = paste("Chapter", i),
+				reportTitleLevel = 1
+			),
+			configFiles[i]
+		)
+	}
+	# create general config file
+	yaml::write_yaml(
+		x = list(config = as.list(basename(configFiles))),
+		file = file.path(configDir, "config.yml")
+	)
+			
+	# example index file
+	indexPath <- file.path(testDir, "index.Rmd")
+	cat(
+		"---",
+		"title: 'Clinical data report'",
+		"output: clinDataReview::gitbook_clinDataReview_report",
+		"---",
+		"# Introduction", 
+		file = indexPath, sep = "\n"
+	)
+	
+	outputDir <- file.path(testDir, "report")
+	output <- render_clinDataReviewReport(
+		configDir = configDir,
+		inputDir = testDir,
+		intermediateDir = file.path(testDir, "interim"),
+		outputDir = outputDir,
+		nCores = min(parallel::detectCores(), 2),
+		quiet = TRUE, # suppress printing of pandoc cmd line
+		verbose = FALSE
+	)
+	
+	# check that all filenames are correct
+	htmlFiles <- list.files(pattern = ".html$", outputDir)
+	htmlFiles <- setdiff(htmlFiles, "404.html")
+	htmlFilesExp <- c(
+		"1-introduction.html", 
+		paste0(2:5, "-chapter-", 1:4, ".html"),
+		"6-appendix.html"
+	)
+	expect_setequal(object = htmlFiles, expected = htmlFilesExp)
+	
+})
+
+test_that("A clinical data report is created successfully in parallel with one chapter containing parallel execution", {
+  
+  skip_on_cran()
+  
+  # fix for: 'Using anchor_sections requires Pandoc 2.0+'
+  skip_if_not(
+	  condition = rmarkdown::pandoc_available(version = "2.0"), 
+	  message = "pandoc 2.0 is not available"
+  )
+  
+  testDir <- tempfile("report")
+  dir.create(testDir)
+  
+  configDir <- file.path(testDir, "config")
+  dir.create(configDir)
+  
+  # create multiple config files
+  nChapters <- 4
+  configFiles <- file.path(configDir, 
+    paste0("config-division-", seq_len(nChapters), ".yml")
+  )
+  for(i in seq_len(nChapters)){
+    yaml::write_yaml(
+      x = list(
+        template = "divisionTemplate.Rmd",
+        templatePackage = "clinDataReview",
+        reportTitle = paste("Chapter", i),
+        reportTitleLevel = 1
+      ),
+      file = configFiles[i]
+    )
+  }
+  
+  # create example chapter run in parallel
+  chapterParallel <- file.path(testDir, "chapter-parallel.Rmd")
+  cat(
+    "# Parallel",
+    "```{r eval = TRUE, echo = FALSE, class.output = \"testParallel\", comment = NA}",
+    "library(parallel)",
+    "cl <- makeCluster(2)",
+    "print(clusterApply(cl, 1:2, get('+'), 3))",
+    "stopCluster(cl = cl)",
+    "```",
+    file = chapterParallel, sep = "\n"
+  )
+  configFileParallel <- file.path(configDir, "config-parallel.yml")
+  yaml::write_yaml(
+    x = list(
+      template = basename(chapterParallel),
+      parallel = TRUE
+    ),
+    file = configFileParallel
+  )
+  configFiles <- c(configFiles, configFileParallel)
+  
+  # create general config file
+  yaml::write_yaml(
+    x = list(config = as.list(basename(configFiles))),
+    file = file.path(configDir, "config.yml")
+  )
+  
+  # example index file
+  indexPath <- file.path(testDir, "index.Rmd")
+  cat(
+    "---",
+    "title: 'Clinical data report'",
+    "output: clinDataReview::gitbook_clinDataReview_report",
+    "---",
+    "# Introduction", 
+    file = indexPath, sep = "\n"
+  )
+  
+  # progress messages during execution
+  outputDir <- file.path(testDir, "report")
+  expect_error(
+	output <- render_clinDataReviewReport(
+		configDir = configDir,
+		inputDir = testDir,
+		intermediateDir = file.path(testDir, "interim"),
+		outputDir = outputDir,
+		nCores = min(parallel::detectCores(), 2),
+		quiet = TRUE, # suppress printing of pandoc cmd line
+		verbose = FALSE
+	),
+    NA
+  )
+  
+  # check that all filenames are correct
+  htmlFiles <- list.files(pattern = "html", outputDir)
+  htmlFiles <- setdiff(htmlFiles, "404.html")
+  htmlFilesExp <- c(
+    "1-introduction.html", 
+    paste0(2:5, "-chapter-", 1:4, ".html"),
+    "6-parallel.html",
+    "7-appendix.html"
+  )
+  expect_setequal(object = htmlFiles, expected = htmlFilesExp)
+
+  # check the content of the chapter run in parallel
+  htmlFileParallel <- xml2::read_xml(x = file.path(outputDir, "6-parallel.html"))
+  headXML <- xml2::xml_find_all(x = htmlFileParallel, xpath = ".//h1[span]")
+  expect_equal(object = xml2::xml_text(headXML), expected = "6 Parallel")
+  
+  codeXML <- xml2::xml_find_all(
+    x = htmlFileParallel, 
+    xpath = ".//pre[contains(@class, 'testParallel')]//code"
+  )
+  expect_match(object = xml2::xml_text(codeXML), regexp = "4.+5")
+  
+})
+
+test_that("A report is correctly split at different chapter-specific levels", {
+			
+	skip_on_cran()
+	
+	# fix for: 'Using anchor_sections requires Pandoc 2.0+'
+	skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+	)
+			
+	testDir <- tempfile("report")
+	dir.create(testDir)
+			
+	# create multiple config files + template
+	template <- file.path(testDir, "chapter.Rmd")
+	cat(
+		"# Chapter",
+		"## Section 1",
+		"### Subsection 1",
+		"### Subsection 2",
+		"## Section 2",
+		file = template, sep = "\n"
+	)
+	configDir <- file.path(testDir, "config")
+	dir.create(configDir)
+	configFiles <- file.path(configDir, 
+		paste0("config-division-", c("chapter", "section", "subsection"), ".yml")
+	)
+	for(i in seq_along(configFiles)){
+		yaml::write_yaml(
+			x = list(
+				template = basename(template),
+				split_by = as.integer(i),
+				reportTitle = paste("Chapter split at level", i)
+			),
+			file = configFiles[i]
+		)
+	}
+	# create general config file
+	yaml::write_yaml(
+		x = list(config = as.list(basename(configFiles))),
+		file = file.path(configDir, "config.yml")
+	)
+			
+	# example index file
+	indexPath <- file.path(testDir, "index.Rmd")
+	cat(
+		"---",
+		"title: 'Clinical data report'",
+		"output:",
+		"  clinDataReview::gitbook_clinDataReview_report:",
+		"    split_by: 'section+number'",
+		"---",
+		"# Introduction", 
+		file = indexPath, sep = "\n"
+	)
+			
+	# progress messages during execution
+	outputDir <- file.path(testDir, "report")
+	output <- render_clinDataReviewReport(
+		configDir = configDir,
+		inputDir = testDir,
+		intermediateDir = file.path(testDir, "interim"),
+		outputDir = outputDir,
+		quiet = TRUE, # suppress printing of pandoc cmd line
+		verbose = FALSE
+	)
+	expect_type(output, "character")
+	expect_match(output, "introduction")
+	
+	# check that all filenames are correct
+	htmlFiles <- list.files(pattern = "html", outputDir)
+	htmlFiles <- setdiff(htmlFiles, "404.html")
+	htmlFilesExp <- c(
+		"1-introduction.html", 
+		# split at chapter level
+		"2-chapter.html", 
+		# split at the section level
+		"3-chapter.html",
+		"3.1-section-1.html", "3.2-section-2.html",
+		# split at the subsection level
+		"4-chapter.html",
+		"4.1-section-1.html", "4.1.1-subsection-1.html", 
+		"4.1.2-subsection-2.html", "4.2-section-2.html",
+		"5-appendix.html",
+		"5.1-session-information.html"
+	)
+	expect_setequal(object = htmlFiles, expected = htmlFilesExp)
+	
+	# check if content is correct
+	getHeaders <- function(file){
+	  x <- xml2::read_xml(x = file)
+		headXML <- xml2::xml_find_all(x = x, xpath = ".//h1[span]|.//h2[span]|.//h3[span]")
+		headers <- xml2::xml_text(headXML)
+		return(headers)
+	}
+	
+	# split at the chapter level
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "2-chapter.html")),
+		expected = c("2 Chapter", "2.1 Section 1", "2.1.1 Subsection 1", 
+			"2.1.2 Subsection 2", "2.2 Section 2")
+	)
+	# split at the section level
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "3-chapter.html")),
+		expected = "3 Chapter"
+	)
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "3.1-section-1.html")),
+		expected = c("3.1 Section 1", "3.1.1 Subsection 1", "3.1.2 Subsection 2")
+	)
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "3.2-section-2.html")),
+		expected = "3.2 Section 2"
+	)
+	
+	# split at the subsection level
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "4-chapter.html")),
+		expected = "4 Chapter"
+	)
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "4.1-section-1.html")),
+		expected = "4.1 Section 1"
+	)
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "4.1.1-subsection-1.html")),
+		expected = "4.1.1 Subsection 1"
+	)
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "4.1.2-subsection-2.html")),
+		expected = "4.1.2 Subsection 2"
+	)
+	expect_setequal(
+		object = getHeaders(file = file.path(outputDir, "4.2-section-2.html")),
+		expected = "4.2 Section 2"
+	)
+	
+})
+
+test_that("The page titles are correctly set for a report", {
+  
+  skip_on_cran()
+  
+  # fix for: 'Using anchor_sections requires Pandoc 2.0+'
+  skip_if_not(
+		condition = rmarkdown::pandoc_available(version = "2.0"), 
+		message = "pandoc 2.0 is not available"
+  )
+  
+  testDir <- tempfile("report")
+  dir.create(testDir)
+  
+  # create multiple config files + template
+  template <- file.path(testDir, "chapter.Rmd")
+  cat(
+    "# Chapter",
+    "## Section 1",
+    "### Subsection 1",
+    "### Subsection 2",
+    "## Section 2",
+    file = template, sep = "\n"
+  )
+  configDir <- file.path(testDir, "config")
+  dir.create(configDir)
+  configFiles <- file.path(configDir, 
+    paste0("config-division-", c("chapter", "section", "subsection"), ".yml")
+  )
+  for(i in seq_along(configFiles)){
+    yaml::write_yaml(
+      x = list(
+        template = basename(template),
+        split_by = as.integer(i),
+        reportTitle = paste("Chapter split at level", i)
+      ),
+      file = configFiles[i]
+    )
+  }
+  # create general config file
+  yaml::write_yaml(
+    x = list(config = as.list(basename(configFiles))),
+    file = file.path(configDir, "config.yml")
+  )
+  
+  # example index file
+  indexPath <- file.path(testDir, "index.Rmd")
+  cat(
+    "---",
+    "title: 'Clinical data report'",
+    "output:",
+    "  clinDataReview::gitbook_clinDataReview_report:",
+    "    split_by: 'section+number'",
+    "---",
+    "# Introduction", 
+    file = indexPath, sep = "\n"
+  )
+  
+  # progress messages during execution
+  outputDir <- file.path(testDir, "report")
+	output <- render_clinDataReviewReport(
+      configDir = configDir,
+      inputDir = testDir,
+      intermediateDir = file.path(testDir, "interim"),
+      outputDir = outputDir,
+      quiet = TRUE, # suppress printing of pandoc cmd line
+	  verbose = FALSE
+    )
+  
+  getTitle <- function(file){
+    x <- xml2::read_xml(x = file)
+    titleXML <- xml2::xml_find_all(x = x, xpath = ".//title")
+    titles <- xml2::xml_text(titleXML)
+    return(titles)
+  }
+  
+  # split at the chapter level
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "2-chapter.html")), 
+    expected = "2 Chapter | Clinical data report"
+  )
+  # split at the section level
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "3-chapter.html")),
+    expected = "3 Chapter | Clinical data report"
+  )
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "3.1-section-1.html")),
+    expected = "3.1 Section 1 | Clinical data report"
+  )
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "3.2-section-2.html")),
+    expected = "3.2 Section 2 | Clinical data report"
+  )
+  
+  # split at the subsection level
+  expect_setequal(
+    object = getTitle(file = file.path(outputDir, "4-chapter.html")),
+    expected = "4 Chapter | Clinical data report"
+  )
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "4.1-section-1.html")),
+    expected = "4.1 Section 1 | Clinical data report"
+  )
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "4.1.1-subsection-1.html")),
+    expected = "4.1.1 Subsection 1 | Clinical data report"
+  )
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "4.1.2-subsection-2.html")),
+    expected = "4.1.2 Subsection 2 | Clinical data report"
+  )
+  expect_equal(
+    object = getTitle(file = file.path(outputDir, "4.2-section-2.html")),
+    expected = "4.2 Section 2 | Clinical data report"
+  )
+  
+})
+
+test_that("The table of contents is correctly set for a report", {
+  
+  skip_on_cran()
+  
+  # fix for: 'Using anchor_sections requires Pandoc 2.0+'
+  skip_if_not(
+		  condition = rmarkdown::pandoc_available(version = "2.0"), 
+		  message = "pandoc 2.0 is not available"
+  )
+  
+  testDir <- tempfile("report")
+  dir.create(testDir)
+  
+  # create multiple config files + template
+  template <- file.path(testDir, "chapter.Rmd")
+  cat(
+    "# Chapter",
+    "## Section 1",
+    "## Section 2",
+    "### Subsection 1",
+    "### Subsection 2", # to test difference of level > 1 with next chapter
+    file = template, sep = "\n"
+  )
+  configDir <- file.path(testDir, "config")
+  dir.create(configDir)
+  configFiles <- file.path(configDir, 
+    paste0("config-division-", c("chapter", "section"), ".yml")
+  )
+  for(i in seq_along(configFiles)){
+    yaml::write_yaml(
+      x = list(
+        template = basename(template),
+        split_by = as.integer(i),
+        reportTitle = paste("Chapter split at level", i)
+      ),
+      file = configFiles[i]
+    )
+  }
+  # create general config file
+  yaml::write_yaml(
+    x = list(config = as.list(basename(configFiles))),
+    file = file.path(configDir, "config.yml")
+  )
+  
+  # example index file
+  indexPath <- file.path(testDir, "index.Rmd")
+  cat(
+    "---",
+    "title: 'Clinical data report'",
+    "output:",
+    "  clinDataReview::gitbook_clinDataReview_report:",
+    "    split_by: 'section+number'",
+    "---",
+    "# Introduction", 
+    file = indexPath, sep = "\n"
+  )
+  
+  # progress messages during execution
+  outputDir <- file.path(testDir, "report")
+    output <- render_clinDataReviewReport(
+      configDir = configDir,
+      inputDir = testDir,
+      intermediateDir = file.path(testDir, "interim"),
+      outputDir = outputDir,
+      quiet = TRUE, # suppress printing of pandoc cmd line
+	  verbose = FALSE
+    )
+  
+	# import introduction and table of contents
+	book <- read_html(x = file.path(outputDir, "1-introduction.html"))
+	toc <- xml_find_all(book, ".//ul[@class='summary']")
+	
+	checkHeader <- function(x, title, link){
+	  head <- xml2::xml_find_first(x, ".//a")
+	  expect_equal(object = xml2::xml_text(head), expected = title)
+	  expect_equal(object = xml2::xml_attr(head, "href"), expected = link)
+	}
+	getSublist <- function(x){
+	  xml_children(xml2::xml_find_first(x, ".//ul"))
+	}
+	
+	tocChapters <- xml2::xml_children(toc)
+	
+	# intro
+	checkHeader(
+	  x = tocChapters[[1]], 
+	  title = "1 Introduction", 
+	  link = "1-introduction.html#introduction"
+  )
+	
+	# chapter split at the chapter level
+	toc2 <- tocChapters[[2]]
+	checkHeader(x = toc2, title = "2 Chapter", link = "2-chapter.html#chapter")
+	toc2Sect <- getSublist(toc2)
+	checkHeader(x = toc2Sect[[1]], title = "2.1 Section 1", 
+	   link = "2-chapter.html#section-1")
+	checkHeader(x = toc2Sect[[2]], title = "2.2 Section 2", 
+	   link = "2-chapter.html#section-2")
+	expect_length(object = getSublist(toc2Sect[[1]]), n = 0)
+	toc22Sub <- getSublist(toc2Sect[[2]])
+	checkHeader(x = toc22Sub[[1]], title = "2.2.1 Subsection 1", 
+	   link = "2-chapter.html#subsection-1")
+	checkHeader(x = toc22Sub[[2]], title = "2.2.2 Subsection 2", 
+	   link = "2-chapter.html#subsection-2")
+	
+	# chapter split at the section level
+	toc3 <- tocChapters[[3]]
+	checkHeader(x = toc3, title = "3 Chapter", 
+	   link = "3-chapter.html#chapter")
+	toc3Sect <- getSublist(toc3)
+	checkHeader(x = toc3Sect[[1]], title = "3.1 Section 1", 
+	   link = "3.1-section-1.html#section-1")
+	checkHeader(x = toc3Sect[[2]], title = "3.2 Section 2", 
+	   link = "3.2-section-2.html#section-2")
+	expect_length(object = getSublist(toc3Sect[[1]]), n = 0)
+	toc32Sub <- getSublist(toc3Sect[[2]])
+	checkHeader(x = toc32Sub[[1]], title = "3.2.1 Subsection 1", 
+	   link = "3.2-section-2.html#subsection-1")
+	checkHeader(x = toc32Sub[[2]], title = "3.2.2 Subsection 2",
+	   link = "3.2-section-2.html#subsection-2")
+
+	# appendix
+	tocAppendix <- tocChapters[[4]]
+	checkHeader(x = tocAppendix, title = "4 Appendix", 
+	   link = "4-appendix.html#appendix")
+	tocAppendixSections <- getSublist(tocAppendix)
+	checkHeader(x = tocAppendixSections, title = "4.1 Session information", 
+	   link = "4.1-session-information.html#session-information")
+	
+})
+
+test_that("The chapter numbers are correctly set when there are more than 10 sections", {
+  
+  secLevels <- c(1, rep(2, 11), rep(3, 2), rep(1, 10))
+  expect_equal(
+    object = clinDataReview:::getTocNumbering(levels = secLevels),
+    expected = c("1", paste0("1.", 1:11), paste0("1.11.", 1:2), as.character(2:11))
+  )
+  
+})
